unit ufrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ComCtrls, ExtCtrls;

type
  TfrmMain = class(TForm)
    stBar: TStatusBar;
    mnuMain: TMainMenu;
    mnuArchivo: TMenuItem;
    mnuSalir: TMenuItem;
    mnuNuevo: TMenuItem;
    sbMain: TScrollBox;
    imgMain: TImage;
    N1: TMenuItem;
    mnuAbrir: TMenuItem;
    mnuGuardar: TMenuItem;
    mnuVer: TMenuItem;
    mnuTools: TMenuItem;
    procedure mnuSalirClick(Sender: TObject);
    procedure mnuNuevoClick(Sender: TObject);
    procedure imgMainMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgMainMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgMainMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure mnuAbrirClick(Sender: TObject);
    procedure mnuGuardarClick(Sender: TObject);
    procedure mnuToolsClick(Sender: TObject);
  private
    { Private declarations }
    FDibujando: Boolean;
    FCentro: TPoint;
    FPuntos: Array of TPoint;
    FAntPuntos: Array of TPoint;
    procedure Rotar(P: TPoint);
    procedure ResetTools;
    procedure SetCentro(const Value: TPoint);
  public
    { Public declarations }
    property Centro: TPoint read FCentro write SetCentro;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses ufrmTools;

procedure TfrmMain.ResetTools;
begin
  with imgMain.Canvas do
  begin
    Pen.Width:= 8;
    frmTools.tbPincel.Position:= Pen.Width;
    Pen.Color:= clBlack;
    frmTools.pnlPincel.Color:= Pen.Color;
    Brush.Color:= clWhite;
    frmTools.pnlRelleno.Color:= Brush.Color;
  end;
  frmTools.tbAncho.Position:= imgMain.Picture.Width;
  frmTools.tbAlto.Position:= imgMain.Picture.Height;
  frmTools.tbDivisiones.Position:= 8;
end;

procedure TfrmMain.Rotar(P: TPoint);
var
  i: Integer;
  a,b,c,d,m: Double;
begin
  a:= P.Y - FCentro.Y;
  b:= FCentro.X - P.X;
  // Calculamos el modulo
  m:= sqrt(sqr(a) + sqr(b));
  // Calculamos el angulo
  if b = 0 then
  begin
    if a < 0 then
      c:= pi/2
    else
      c:= 3*pi/2;
  end else
  begin
    c:= Arctan(a/b);
    if c < 0 then
      c:= c + (2*pi);
    if b > 0 then
      c:= c + pi;
    if c > 2*pi then
      c:= c - (2*pi);
  end;
  SetLength(FPuntos,frmTools.tbDivisiones.Position);
  d:= (2*pi) / frmTools.tbDivisiones.Position;
  for i:= 0 to frmTools.tbDivisiones.Position - 1 do
  begin
    c:= c + d;
    FPuntos[i].X:= Trunc(m * cos(c)) +  FCentro.X;
    FPuntos[i].Y:= FCentro.Y - Trunc(m * sin(c));
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // Inicializamos las variables
  Setlength(FPuntos,0);
  Setlength(FAntPuntos,0);
  // Propiedades
  DoubleBuffered:= TRUE;
end;

procedure TfrmMain.imgMainMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
 var
  i: Integer;
begin
  if ssLeft in Shift then
    with imgMain.Canvas do
    begin
      Pen.Width:= frmTools.tbPincel.Position;
      Pen.Color:= frmTools.pnlPincel.Color;
      Brush.Color:= frmTools.pnlRelleno.Color;
      Rotar(Point(X,Y));
      SetLength(FAntPuntos,Length(FPuntos));
      for i:= 0 to Length(FPuntos) - 1 do
        FAntPuntos[i]:= FPuntos[i];
      if frmTools.btnPincel.Down then
      begin
        FDibujando:= TRUE;
      end else
        for i:= 0 to frmTools.tbDivisiones.Position - 1 do
          FloodFill(FPuntos[i].X,FPuntos[i].Y,
            Pixels[FPuntos[i].X,FPuntos[i].Y],fsSurface);
    end;
end;

procedure TfrmMain.imgMainMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  i: Integer;
begin
  if ssLeft in Shift then
  begin
    with imgMain.Canvas do
    begin
      Rotar(Point(X,Y));
      if Length(FAntPuntos) <> Length (FPuntos) then
        Exit;
      if frmTools.btnPincel.Down then
        for i:= 0 to Length(FPuntos) - 1 do
        begin
          MoveTo(FAntPuntos[i].X,FAntPuntos[i].Y);
          LineTo(FPuntos[i].X,FPuntos[i].Y);
        end;
      for i:= 0 to Length(FPuntos) - 1 do
        FAntPuntos[i]:= FPuntos[i];
    end;
  end else
    FDibujando:= FALSE;
  stBar.Panels[0].Text := Format('X: %d Y: %d',[X,Y]);
  stBar.Panels[1].Text := Format('Distancia: %.2f',[sqrt(sqr(X-FCentro.X) +
    sqr(X-FCentro.X))]);
end;

procedure TfrmMain.imgMainMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDibujando:= FALSE;
  Setlength(FPuntos,0);
  Setlength(FAntPuntos,0);
end;

procedure TfrmMain.mnuAbrirClick(Sender: TObject);
begin
  with TOpenDialog.Create(nil) do
  try
    Filter:= 'Bitmaps|*.bmp';
    if Execute then
    begin
      imgMain.Picture.LoadFromFile(FileName);
      Centro:= Point(imgMain.Picture.Width div 2,imgMain.Picture.Height div 2);
      frmTools.tbAncho.Position:= imgMain.Picture.Width;
      frmTools.tbAlto.Position:= imgMain.Picture.Height;
    end;
  finally
    Free;
  end;
end;

procedure TfrmMain.mnuGuardarClick(Sender: TObject);
begin
  with TSaveDialog.Create(nil) do
  try
    Filter:= 'Bitmaps|*.bmp';
    if Execute then
      imgMain.Picture.SaveToFile(Filename);
  finally
    Free;
  end;
end;

procedure TfrmMain.mnuNuevoClick(Sender: TObject);
var
  Bitmap: TBitmap;
begin
  imgMain.Picture:= nil;
  Bitmap:= TBitmap.Create;
  try
    Bitmap.Canvas.Brush.Color:= clWhite;
    Bitmap.Width:= 800;
    Bitmap.Height:= 600;
    imgMain.Picture.Assign(Bitmap);
    Centro:= Point(Bitmap.Width div 2,Bitmap.Height div 2);
  finally
    Bitmap.Free;
  end;
  ResetTools;
end;

procedure TfrmMain.mnuSalirClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.mnuToolsClick(Sender: TObject);
begin
  frmTools.Visible:= TRUE;
end;

procedure TfrmMain.SetCentro(const Value: TPoint);
begin
  FCentro := Value;
end;

end.
